import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.junit.Test;

/**
 * CIS 1210, Explosive Debugging
 *
 * @author blank, 22su
 * @author huangtif, 22fa
 */
public class GradeHistogramTests {
    @Test
    public void testAddGet() {
        GradeHistogram h = new GradeHistogram();
        h.addGrade(1);
        h.addGrade(1);
        h.addGrade(1);
        h.addGrade(1);
        h.addGrade(1);
        h.addGrade(1);
        assertEquals(6, h.getFrequency(1));
    }

    @Test
    public void stressTestAddGet() {
        List<Integer> correct = new ArrayList<>();
        for (int i = 0; i < 10; i++) {
            correct.add(0);
        }

        GradeHistogram h = new GradeHistogram();
        Random r = new Random(1337);
        for (int i = 0; i < 10000; i++) {
            int x = r.nextInt(10);
            h.addGrade(x);
            correct.set(x, correct.get(x) + 1);

            for (int j = 0; j < 10; j++) {
                assertEquals((int) correct.get(j), h.getFrequency(j));
            }
        }
    }

    @Test
    public void testToString() {
        GradeHistogram h = new GradeHistogram();
        for (int i = 0; i < 10; i++) {
            for (int j = 0; j < i; j++) {
                h.addGrade(i);
            }
        }
        assertEquals(
            "         *\n" +
            "        **\n" +
            "       ***\n" +
            "      ****\n" +
            "     *****\n" +
            "    ******\n" +
            "   *******\n" +
            "  ********\n" +
            " *********\n", h.toString());
    }

    @Test
    public void testGradeRange() {
        GradeHistogram h = new GradeHistogram();
        for (int i = -100; i < 100; i++) {
            int grade = i;
            if (0 <= grade && grade < 10) {
                h.addGrade(grade); // Do not throw
            } else {
                assertThrows(IllegalArgumentException.class, () -> h.addGrade(grade));
            }
        }
        for (int i = -100; i < 100; i++) {
            int grade = i;
            if (0 <= grade && grade < 10) {
                assertEquals(1, h.getFrequency(i));
            } else {
                assertThrows(IllegalArgumentException.class, () -> h.getFrequency(grade));
            }
        }
    }
}
